#include <iostream>
#include <mutex>
//#include <path-spec>
#include <mysql/mysql.h>
//#include <mysql.h>
#include <mysql_driver.h>
#include <mysql_connection.h>
#include <cppconn/statement.h>
#include <cppconn/resultset.h>

class Database {
public:
    static Database& getInstance() {
        static Database instance; // Инициализация одиночки
        return instance;
    }

    bool connect(const char* host, const char* user, const char* password, const char* db_name) {
        conn = mysql_init(nullptr);
        if (conn == nullptr) {
            std::cerr << "mysql_init() failed";
            return false;
        }

        if (mysql_real_connect(conn, host, user, password, db_name, 0, nullptr, 0) == nullptr) {
            std::cerr << "mysql_real_connect() failed";
            mysql_close(conn);
            return false;
        }
        return true;
    }

    void executeQuery(const char* query) {
        if (mysql_query(conn, query)) {
            std::cerr << "QUERY FAILED: " << mysql_error(conn) << std::endl;
        } else {
            MYSQL_RES *res = mysql_store_result(conn);
            if (res) {
                MYSQL_ROW row;
                while ((row = mysql_fetch_row(res))) {
                    for (unsigned int i = 0; i < mysql_num_fields(res); i++) {
                        std::cout << (row[i] ? row[i] : "NULL") << "\t";
                    }
                    std::cout << std::endl;
                }
                mysql_free_result(res);
            }
        }
    }

    ~Database() {
        if (conn) {
            mysql_close(conn);
        }
    }

private:
    Database() : conn(nullptr) {} // Приватный конструктор
    Database(const Database&) = delete; // Запрет копирования
    Database& operator=(const Database&) = delete; // Запрет присваивания

    MYSQL *conn;
};


int mariadb() {
    Database& db = Database::getInstance();
    sqlDB sql_init_struct;
    if (!db.connect(sql_init_struct.sql_host.c_str(), sql_init_struct.sql_login.c_str(), sql_init_struct.sql_passwd.c_str(), sql_init_struct.sql_db.c_str())) {
        return EXIT_FAILURE;
    }

    // Пример запроса к базе данных
    db.executeQuery("SELECT * FROM your_table;");
    
    return EXIT_SUCCESS;
}
